# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Utilities for the unit tests."""

PROJECT = {
    "custom_keys": [],
    "description": "Fedora-messaging schemas for fedocal",
    "full_url": "http://localhost.localdomain/fedora-infra/fedocal-messages",
    "parent": None,
    "date_modified": "1602658714",
    "access_users": {
        "admin": [],
        "collaborator": [],
        "ticket": [],
        "owner": ["pingou"],
        "commit": [],
    },
    "namespace": "fedora-infra",
    "priorities": {},
    "id": 8820,
    "access_groups": {"admin": [], "collaborator": [], "ticket": [], "commit": []},
    "milestones": {},
    "user": {
        "fullname": "Pierre-YvesChibon",
        "url_path": "user/pingou",
        "full_url": "http://localhost.localdomain/user/pingou",
        "name": "pingou",
    },
    "date_created": "1602658714",
    "fullname": "fedora-infra/fedocal-messages",
    "url_path": "fedora-infra/fedocal-messages",
    "close_status": [],
    "tags": [],
    "name": "fedocal-messages",
}

FORK = {
    "custom_keys": [],
    "description": "A git centered forge",
    "full_url": "http://localhost.localdomain/fork/jjames/pagure",
    "parent": {
        "custom_keys": [],
        "description": "A git centered forge",
        "full_url": "http://localhost.localdomain/pagure",
        "parent": None,
        "date_modified": "1590422384",
        "access_users": {
            "admin": ["ryanlerch"],
            "collaborator": [],
            "ticket": [
                "farhaan",
                "jcline",
                "jlanda",
                "karsten",
                "lenkaseg",
                "lslebodn",
                "mprahl",
                "ngompa",
                "vivekanand1101",
            ],
            "owner": ["pingou"],
            "commit": ["bkabrda", "cverna", "puiterwijk"],
        },
        "namespace": None,
        "priorities": {"": "", "1": "High", "3": "Low", "2": "Normal"},
        "id": 10,
        "access_groups": {
            "admin": [],
            "collaborator": [],
            "ticket": [],
            "commit": [],
        },
        "milestones": {
            "5.0": {"active": False, "date": None},
            "5.1": {"active": False, "date": None},
            "5.2": {"active": False, "date": "January 7th 2019"},
            "5.3": {"active": False, "date": "February 7th 2019"},
            "5.4": {"active": False, "date": "March 7th 2019"},
            "5.5": {"active": False, "date": "April 7th 2019"},
            "5.6": {"active": False, "date": "May 7th 2019"},
            "5.7": {"active": False, "date": "July 7th 2019"},
            "5.8": {"active": False, "date": "August 7th"},
            "5.9": {"active": False, "date": None},
            "Coming 3 months": {"active": True, "date": None},
            "5.12": {"active": True, "date": None},
            "5.13": {"active": True, "date": None},
            "5.10": {"active": False, "date": None},
            "5.11": {"active": False, "date": None},
            "6.0": {"active": True, "date": None},
            "Coming 6 months": {"active": True, "date": None},
        },
        "user": {
            "fullname": "Pierre-YvesChibon",
            "url_path": "user/pingou",
            "full_url": "http://localhost.localdomain/user/pingou",
            "name": "pingou",
        },
        "date_created": "1431549490",
        "fullname": "pagure",
        "url_path": "pagure",
        "close_status": [
            "Invalid",
            "Insufficient data",
            "Fixed",
            "Duplicate",
            "Won't Fix",
        ],
        "tags": ["pagure", "fedmsg"],
        "name": "pagure",
    },
    "date_modified": "1602543006",
    "access_users": {
        "admin": [],
        "collaborator": [],
        "ticket": [],
        "owner": ["jjames"],
        "commit": [],
    },
    "namespace": None,
    "priorities": {},
    "id": 8813,
    "access_groups": {
        "admin": [],
        "collaborator": [],
        "ticket": [],
        "commit": [],
    },
    "milestones": {},
    "user": {
        "fullname": "Jerry James",
        "url_path": "user/jjames",
        "full_url": "http://localhost.localdomain/user/jjames",
        "name": "jjames",
    },
    "date_created": "1602543006",
    "fullname": "forks/jjames/pagure",
    "url_path": "fork/jjames/pagure",
    "close_status": [],
    "tags": [],
    "name": "pagure",
}


ISSUE = {
    "status": "Open",
    "priority": 3,
    "last_updated": "1602574891",
    "blocks": [],
    "boards": [
        {
            "status": {
                "default": False,
                "bg_color": "#15d415",
                "close_status": "Fixed",
                "name": "Done",
                "close": True,
            },
            "board": {
                "active": True,
                "status": [
                    {
                        "default": True,
                        "bg_color": "#ffb300",
                        "close_status": None,
                        "name": "Backlog",
                        "close": False,
                    },
                    {
                        "default": False,
                        "bg_color": "#f4ff64",
                        "close_status": None,
                        "name": "Triaged",
                        "close": False,
                    },
                    {
                        "default": False,
                        "bg_color": "#99d200",
                        "close_status": None,
                        "name": "In Progress",
                        "close": False,
                    },
                    {
                        "default": False,
                        "bg_color": "#3c7bff",
                        "close_status": None,
                        "name": "In Review",
                        "close": False,
                    },
                    {
                        "default": False,
                        "bg_color": "#15d415",
                        "close_status": "Fixed",
                        "name": "Done",
                        "close": True,
                    },
                    {
                        "default": False,
                        "bg_color": "#e80909",
                        "close_status": None,
                        "name": "Blocked",
                        "close": False,
                    },
                ],
                "tag": {
                    "tag_color": "#3efa0e",
                    "tag": "ops",
                    "tag_description": "ops problem now...",
                },
                "name": "ops",
                "full_url": "http://localhost.localdomain/test/boards/ops",
            },
            "rank": 8,
        }
    ],
    "tags": ["medium-gain", "medium-trouble", "ops"],
    "title": "aws: fedora-ci access to s3 bucket request",
    "related_prs": [],
    "milestone": None,
    "comments": [
        {
            "comment": "This is related to the previous ticket and comment: "
            "https://pagure.io/fedora-infrastructure/issue/8958#comment-656531",
            "reactions": {},
            "parent": None,
            "notification": False,
            "edited_on": None,
            "editor": None,
            "date_created": "1599751495",
            "id": 677658,
            "user": {
                "fullname": "Andrei Stepanov",
                "url_path": "user/astepano",
                "full_url": "http://localhost.localdomain/user/astepano",
                "name": "astepano",
            },
        },
        {
            "comment": "**Metadata Update from @kevin**:\n- Issue priority "
            "set to: Waiting on Assignee (was: Needs Review)\n- "
            "Issue tagged with: medium-gain, medium-trouble, ops",
            "reactions": {},
            "parent": None,
            "notification": True,
            "edited_on": None,
            "editor": None,
            "date_created": "1599760908",
            "id": 677717,
            "user": {
                "fullname": "Kevin Fenzi",
                "url_path": "user/kevin",
                "full_url": "http://localhost.localdomain/user/kevin",
                "name": "kevin",
            },
        },
    ],
    "close_status": None,
    "content": "Good day.\r\n\r\nIn context of fedora-ci initiative may we ask "
    "to have an access to s3 bucket please?\r\nCurrently we are deploying "
    "pipelines on AWS EKS: https://osci-jenkins-1.ci.fedoraproject.org/\r\n"
    "We need to store artifacts and logs with easy way of maintenance. \r\n"
    "\r\nI see there are two ways:\r\n\r\n1. Allow to `user/fedora-ci-osci` "
    "token create s3 buckets on demand. This would be preferable solution. "
    "From our side I promise to follow resource-tagging conventions... ",
    "assignee": None,
    "depends": [],
    "private": False,
    "date_created": "1599751343",
    "full_url": "http://localhost.localdomain/test/issue/9311",
    "id": 9311,
    "closed_at": "1602535496",
    "closed_by": {
        "fullname": "Kevin Fenzi",
        "url_path": "user/kevin",
        "full_url": "http://localhost.localdomain/user/kevin",
        "name": "kevin",
    },
    "custom_fields": [],
    "user": {
        "fullname": "Andrei Stepanov",
        "full_url": "http://localhost.localdomain/user/astepano",
        "url_path": "user/astepano",
        "name": "astepano",
    },
}


PULL_REQUEST = {
    "last_updated": "1602575128",
    "uid": "f763d8f1254440c782a7cde84ae5f422",
    "initial_comment": "I just created a new project on pagure.io.  "
    "Two of the instructions I was given on setting up my new repo were out of order.",
    "commit_stop": "2e8db9ce332cae0747f6b38e414bb99634f9b662",
    "remote_git": None,
    "closed_at": "1602575128",
    "full_url": "http://localhost.localdomain/pagure/pull-request/5014",
    "id": 5014,
    "title": "Reverse out of order instructions for new repos",
    "comments": [
        {
            "comment": "Good catch, thanks!",
            "reactions": {},
            "parent": None,
            "notification": False,
            "tree": None,
            "filename": None,
            "edited_on": None,
            "editor": None,
            "date_created": "1602575085",
            "commit": None,
            "line": None,
            "id": 133903,
            "user": {
                "fullname": "Pierre-YvesChibon",
                "url_path": "user/pingou",
                "full_url": "http://localhost.localdomain/user/pingou",
                "name": "pingou",
            },
        },
        {
            "comment": "Pull-Request has been merged by pingou",
            "reactions": {},
            "parent": None,
            "notification": True,
            "tree": None,
            "filename": None,
            "edited_on": None,
            "editor": None,
            "date_created": "1602575128",
            "commit": None,
            "line": None,
            "id": 133905,
            "user": {
                "fullname": "Pierre-YvesChibon",
                "url_path": "user/pingou",
                "full_url": "http://localhost.localdomain/user/pingou",
                "name": "pingou",
            },
        },
    ],
    "branch": "master",
    "status": "Merged",
    "tags": [],
    "user": {
        "fullname": "Jerry James",
        "url_path": "user/jjames",
        "full_url": "http://localhost.localdomain/user/jjames",
        "name": "jjames",
    },
    "date_created": "1602543367",
    "closed_by": {
        "fullname": "Pierre-YvesChibon",
        "url_path": "user/pingou",
        "full_url": "http://localhost.localdomain/user/pingou",
        "name": "pingou",
    },
    "branch_from": "master",
    "assignee": None,
    "commit_start": "2e8db9ce332cae0747f6b38e414bb99634f9b662",
    "project": {
        "custom_keys": [],
        "description": "A git centered forge",
        "full_url": "http://localhost.localdomain/pagure",
        "parent": None,
        "date_modified": "1590422384",
        "access_users": {
            "admin": ["ryanlerch"],
            "collaborator": [],
            "ticket": [
                "farhaan",
                "jcline",
                "jlanda",
                "karsten",
                "lenkaseg",
                "lslebodn",
                "mprahl",
                "ngompa",
                "vivekanand1101",
            ],
            "owner": ["pingou"],
            "commit": ["bkabrda", "cverna", "puiterwijk"],
        },
        "namespace": None,
        "priorities": {"": "", "1": "High", "3": "Low", "2": "Normal"},
        "id": 10,
        "access_groups": {
            "admin": [],
            "collaborator": [],
            "ticket": [],
            "commit": [],
        },
        "milestones": {
            "5.0": {"active": False, "date": None},
            "5.1": {"active": False, "date": None},
            "5.2": {"active": False, "date": "January 7th 2019"},
            "5.3": {"active": False, "date": "February 7th 2019"},
            "5.4": {"active": False, "date": "March 7th 2019"},
            "5.5": {"active": False, "date": "April 7th 2019"},
            "5.6": {"active": False, "date": "May 7th 2019"},
            "5.7": {"active": False, "date": "July 7th 2019"},
            "5.8": {"active": False, "date": "August 7th"},
            "5.9": {"active": False, "date": None},
            "Coming 3 months": {"active": True, "date": None},
            "5.12": {"active": True, "date": None},
            "5.13": {"active": True, "date": None},
            "5.10": {"active": False, "date": None},
            "5.11": {"active": False, "date": None},
            "6.0": {"active": True, "date": None},
            "Coming 6 months": {"active": True, "date": None},
        },
        "user": {
            "fullname": "Pierre-YvesChibon",
            "url_path": "user/pingou",
            "full_url": "http://localhost.localdomain/user/pingou",
            "name": "pingou",
        },
        "date_created": "1431549490",
        "fullname": "pagure",
        "url_path": "pagure",
        "close_status": [
            "Invalid",
            "Insufficient data",
            "Fixed",
            "Duplicate",
            "Won't Fix",
        ],
        "tags": ["pagure", "fedmsg"],
        "name": "pagure",
    },
    "repo_from": {
        "custom_keys": [],
        "description": "A git centered forge",
        "parent": {
            "custom_keys": [],
            "description": "A git centered forge",
            "full_url": "http://localhost.localdomain/pagure",
            "parent": None,
            "date_modified": "1590422384",
            "access_users": {
                "admin": ["ryanlerch"],
                "collaborator": [],
                "ticket": [
                    "farhaan",
                    "jcline",
                    "jlanda",
                    "karsten",
                    "lenkaseg",
                    "lslebodn",
                    "mprahl",
                    "ngompa",
                    "vivekanand1101",
                ],
                "owner": ["pingou"],
                "commit": ["bkabrda", "cverna", "puiterwijk"],
            },
            "namespace": None,
            "priorities": {"": "", "1": "High", "3": "Low", "2": "Normal"},
            "id": 10,
            "access_groups": {
                "admin": [],
                "collaborator": [],
                "ticket": [],
                "commit": [],
            },
            "milestones": {
                "5.0": {"active": False, "date": None},
                "5.1": {"active": False, "date": None},
                "5.2": {"active": False, "date": "January 7th 2019"},
                "5.3": {"active": False, "date": "February 7th 2019"},
                "5.4": {"active": False, "date": "March 7th 2019"},
                "5.5": {"active": False, "date": "April 7th 2019"},
                "5.6": {"active": False, "date": "May 7th 2019"},
                "5.7": {"active": False, "date": "July 7th 2019"},
                "5.8": {"active": False, "date": "August 7th"},
                "5.9": {"active": False, "date": None},
                "Coming 3 months": {"active": True, "date": None},
                "5.12": {"active": True, "date": None},
                "5.13": {"active": True, "date": None},
                "5.10": {"active": False, "date": None},
                "5.11": {"active": False, "date": None},
                "6.0": {"active": True, "date": None},
                "Coming 6 months": {"active": True, "date": None},
            },
            "user": {
                "fullname": "Pierre-YvesChibon",
                "url_path": "user/pingou",
                "full_url": "http://localhost.localdomain/user/pingou",
                "name": "pingou",
            },
            "date_created": "1431549490",
            "fullname": "pagure",
            "url_path": "pagure",
            "close_status": [
                "Invalid",
                "Insufficient data",
                "Fixed",
                "Duplicate",
                "Won't Fix",
            ],
            "tags": ["pagure", "fedmsg"],
            "name": "pagure",
        },
        "date_modified": "1602543006",
        "access_users": {
            "admin": [],
            "collaborator": [],
            "ticket": [],
            "owner": ["jjames"],
            "commit": [],
        },
        "namespace": None,
        "priorities": {},
        "id": 8813,
        "access_groups": {
            "admin": [],
            "collaborator": [],
            "ticket": [],
            "commit": [],
        },
        "milestones": {},
        "user": {
            "fullname": "Jerry James",
            "url_path": "user/jjames",
            "name": "jjames",
        },
        "date_created": "1602543006",
        "fullname": "forks/jjames/pagure",
        "url_path": "fork/jjames/pagure",
        "close_status": [],
        "tags": [],
        "name": "pagure",
    },
    "cached_merge_status": "unknown",
    "updated_on": "1602575128",
    "threshold_reached": None,
}


COMMIT_FLAG = {
    "comment": "Build failed.",
    "status": "failure",
    "date_updated": "1602688864",
    "percent": "0",
    "username": "jenkins",
    "url": "https://jenkins.example.org/job/project/job/project-postmerge/264/",
    "commit_hash": "36ca6c643221858eda70e2b52a6aee666dc6e576",
    "date_created": "1602687207",
    "user": {
        "fullname": "foobar",
        "url_path": "user/foobar",
        "name": "foobar",
        "full_url": "http://localhost.localdomain/user/foobar",
    },
}


GROUP = {
    "display_name": "fedora infra folks",
    "description": "Fedora infrastructure team",
    "creator": {
        "fullname": "pingou",
        "url_path": "user/pingou",
        "name": "pingou",
        "full_url": "http://localhost.localdomain/user/pingou",
    },
    "members": [
        "pingou",
        "kevin",
        "smooge",
        "ralph",
    ],
    "date_created": "1598094647",
    "group_type": "user",
    "name": "fedora-infra",
    "full_url": "http://localhost.localdomain/group/fedora-infra",
}



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from .base import (
    COMMIT_FLAG,
    IssueOrPullRequestMessage,
    PROJECT,
    PULL_REQUEST,
    SCHEMA_URL,
)


class PullRequestMessage(IssueOrPullRequestMessage):
    object_type = "pullrequest"


class PullRequestAssignedAddedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a pull request is assigned.
    """

    topic = "pagure.pull-request.assigned.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
            "project": PROJECT,
        },
        "required": ["agent", "pullrequest", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Pull-request {name}#{id} was assigned\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} assigned the pull-request {name}#{id} to {assignee}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
            assignee=self.body["pullrequest"]["assignee"]["name"],
        )


class PullRequestAssignedResetV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a pull request is un-assigned.
    """

    topic = "pagure.pull-request.assigned.reset"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
            "project": PROJECT,
        },
        "required": ["agent", "pullrequest", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Pull-request {name}#{id} was un-assigned\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} reset the assignee of the pull-request {name}#{id}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
        )


class PullRequestClosedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a pull request is closed.
    """

    topic = "pagure.pull-request.closed"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
            "merged": {"type": "boolean"},
        },
        "required": ["agent", "pullrequest", "merged"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Pull-Request: {fullname}#{id} has been {action}\nBy: {agent_name}".format(
                fullname=self.body["pullrequest"]["project"]["fullname"],
                id=self.body["pullrequest"]["id"],
                agent_name=self.agent_name,
                action="merged" if self.body["merged"] else "closed without merging",
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} {action} the pull-request {name}#{id}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            action="merged" if self.body["merged"] else "closed without merging",
        )


class PullRequestCommentAddedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a comment is added to a PR.
    """

    topic = "pagure.pull-request.comment.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
        },
        "required": ["agent", "pullrequest"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Pull-Request: {fullname}#{id} has a new comment\nBy: {agent_name}".format(
                fullname=self.body["pullrequest"]["project"]["fullname"],
                id=self.body["pullrequest"]["id"],
                agent_name=self.agent_name,
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} commented on the pull-request {name}#{id}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def url(self):
        full_url = self.body["pullrequest"]["full_url"]
        commentid = self.body["pullrequest"]["comments"][-1]["id"]

        return "{full_url}#comment-{commentid}".format(
            full_url=full_url, commentid=commentid
        )


class PullRequestCommentEditedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a comment is edited on a PR.
    """

    topic = "pagure.pull-request.comment.edited"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
        },
        "required": ["agent", "pullrequest"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Edited comment on Pull-Request: {fullname}#{id}\nBy: {agent_name}".format(
                fullname=self.body["pullrequest"]["project"]["fullname"],
                id=self.body["pullrequest"]["id"],
                agent_name=self.agent_name,
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} edited comment on the pull-request {name}#{id}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def url(self):
        url = self.body["pullrequest"]["full_url"]
        comments = self.body["pullrequest"]["comments"]
        if comments:
            commentid = self.body["pullrequest"]["comments"][-1]["id"]
            url = f"{url}#comment-{commentid}"
        return url


class PullRequestFlagAddedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a flag is added on a PR.
    """

    topic = "pagure.pull-request.flag.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
            "flag": COMMIT_FLAG,
        },
        "required": ["agent", "pullrequest", "flag"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "New pull-request flag: {username} {status}\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            username=self.body["flag"]["username"],
            status=self.body["flag"]["status"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "Pull-request {name}#{id} was flagged as {status} by {username}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.body["flag"]["username"],
            status=self.body["flag"]["status"],
        )


class PullRequestFlagUpdatedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a flag is updated on a PR
    """

    topic = "pagure.pull-request.flag.updated"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
            "flag": COMMIT_FLAG,
        },
        "required": ["agent", "pullrequest", "flag"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Pull-request flag updated: {username} {status}\nBy: {agent_name}".format(
                agent_name=self.agent_name,
                username=self.body["flag"]["username"],
                status=self.body["flag"]["status"],
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} updated flag on pull-request {name}#{id} to {status}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.body["flag"]["username"],
            status=self.body["flag"]["status"],
        )


class PullRequestInitialCommentEditedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an initial PR comment is edited.
    """

    topic = "pagure.pull-request.initial_comment.edited"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "pullrequest": PULL_REQUEST,
        },
        "required": ["agent", "pullrequest", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Description of pull-request {name}#{id} edited\nBy: {agent_name}".format(
                agent_name=self.agent_name,
                name=self.body["pullrequest"]["project"]["fullname"],
                id=self.body["pullrequest"]["id"],
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} has edited the description of the pull-request {name}#{id}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
        )


class PullRequestNewV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a pull request is created.
    """

    topic = "pagure.pull-request.new"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
        },
        "required": ["agent", "pullrequest"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "New Pull-Request: {fullname}#{id}\nBy: {agent_name}".format(
            fullname=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} opened a pull-request {fullname}#{id}: {title}".format(
            agent_name=self.agent_name,
            fullname=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            title=self.body["pullrequest"]["title"],
        )


class PullRequestRebasedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a PR is rebased.
    """

    topic = "pagure.pull-request.rebased"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
        },
        "required": ["agent", "pullrequest"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Pull-request {name}#{id} was rebased\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} rebased the pull-request {name}#{id}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
        )


class PullRequestReopenedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a PR is reopened.
    """

    topic = "pagure.pull-request.reopened"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
        },
        "required": ["agent", "pullrequest"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Pull-request {name}#{id} was re-opened\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} re-opened the pull-request {name}#{id}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
        )


class PullRequestTagAddedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a tag is added on a PR.
    """

    topic = "pagure.pull-request.tag.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
            "project": PROJECT,
            "tags": {"type": "array", "items": {"type": "string"}},
        },
        "required": ["agent", "pullrequest", "project", "tags"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Pull-request {name}#{id} was tagged\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} tagged the pull-request {name}#{id} with {tags}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
            tags=", ".join(self.body["tags"]),
        )


class PullRequestTagRemovedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a tag is removed on a PR.
    """

    topic = "pagure.pull-request.tag.removed"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
            "project": PROJECT,
            "tags": {"type": "array", "items": {"type": "string"}},
        },
        "required": ["agent", "pullrequest", "project", "tags"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Pull-request {name}#{id} was un-tagged\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} un-tagged the pull-request {name}#{id} with: {tags}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
            tags=", ".join(self.body["tags"]),
        )


class PullRequestUpdatedV1(PullRequestMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a PR is updated.
    """

    topic = "pagure.pull-request.updated"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "pullrequest": PULL_REQUEST,
        },
        "required": ["agent", "pullrequest"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Pull-request {name}#{id} was updated\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{username} updated the pull-request {name}#{id}".format(
            name=self.body["pullrequest"]["project"]["fullname"],
            id=self.body["pullrequest"]["id"],
            username=self.agent_name,
        )



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Unit tests for the message schema."""

from jsonschema import ValidationError

import pytest

from pagure_messages.git_schema import GitReceiveV1

from .utils import PROJECT


@pytest.fixture
def body():
    return {
        "agent": "dummy-user",
        "forced": False,
        "repo": PROJECT,
        "old_commit": "hash_commit_old",
        "branch": "refs/heads/develop",
        "authors": [
            {
                "fullname": "Dummy User",
                "url_path": "user/dummy-user",
                "name": "dummy-user",
                "email": "dummy-user@example.com",
            }
        ],
        "total_commits": 42,
        "start_commit": "hash_commit_start",
        "end_commit": "hash_commit_stop",
    }


def test_minimal(body):
    """
    Assert the message schema validates a message with the required fields.
    """
    message = GitReceiveV1(body=body)
    message.validate()
    assert message.url == (
        "http://localhost.localdomain/fedora-infra/fedocal-messages"
        "/c/hash_commit_old..hash_commit_stop"
    )
    assert message.packages == []
    assert message.containers == []
    assert message.modules == []
    assert message.flatpaks == []


def test_minimal_short_branch(body):
    """
    Assert the message schema validates a message with the required fields.
    """
    body["branch"] = "develop"
    message = GitReceiveV1(body=body)
    message.validate()
    assert message.url == (
        "http://localhost.localdomain/fedora-infra/fedocal-messages/"
        "c/hash_commit_old..hash_commit_stop"
    )


def test_missing_fields(body):
    """Assert an exception is actually raised on validation failure."""
    del body["agent"]
    message = GitReceiveV1(body=body)
    with pytest.raises(ValidationError):
        message.validate()


def test_str(body):
    """Assert __str__ produces a human-readable message."""
    expected_str = (
        "Dummy User (dummy-user) pushed 42 commits on "
        "fedora-infra/fedocal-messages.\n"
        "Branch: develop\n"
    )
    message = GitReceiveV1(body=body)
    message.validate()
    assert expected_str == str(message)


def test_summary(body):
    """Assert the summary is correct."""
    expected_summary = (
        "dummy-user pushed 42 commits on "
        "fedora-infra/fedocal-messages (branch: develop)"
    )
    message = GitReceiveV1(body=body)
    message.validate()
    assert expected_summary == message.summary


def test_url_one_commit(body):
    """
    Assert the URL is correct when a single commit was received.
    """
    body["total_commits"] = 1
    message = GitReceiveV1(body=body)
    message.validate()
    assert message.url == (
        "http://localhost.localdomain/fedora-infra/fedocal-messages"
        "/c/hash_commit_stop"
    )


def test_patch_url(body):
    """
    Assert the patch URL is correct when a single commit was received.
    """
    body["total_commits"] = 1
    message = GitReceiveV1(body=body)
    message.validate()
    assert message.patch_url == (
        "http://localhost.localdomain/fedora-infra/fedocal-messages"
        "/c/hash_commit_stop.patch"
    )


def test_patch_url_multiple_commits(body):
    """
    Assert the patch URL is correct when a single commit was received.
    """
    message = GitReceiveV1(body=body)
    message.validate()
    assert message.patch_url is None


@pytest.mark.parametrize(
    "namespace,msg_attr",
    [
        ("rpms", "packages"),
        ("containers", "containers"),
        ("modules", "modules"),
        ("flatpaks", "flatpaks"),
    ],
)
def test_artifacts(namespace, msg_attr):
    """
    Assert the message has the correct artifacts set
    """
    body = {
        "agent": "dummy-user",
        "forced": False,
        "repo": PROJECT.copy(),
        "old_commit": "hash_commit_old",
        "branch": "refs/heads/develop",
        "authors": [
            {
                "fullname": "dummy-user",
                "url_path": "user/dummy-user",
                "name": "dummy-user",
                "email": None,
            }
        ],
        "total_commits": 42,
        "start_commit": "hash_commit_start",
        "end_commit": "hash_commit_stop",
    }
    body["repo"]["namespace"] = namespace
    message = GitReceiveV1(body=body)

    for test_attr in ("packages", "containers", "modules", "flatpaks"):
        expected = ["fedocal-messages"] if msg_attr == test_attr else []
        assert getattr(message, test_attr) == expected



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from .base import (
    ISSUE,
    IssueOrPullRequestMessage,
    PROJECT,
    SCHEMA_URL,
)


class IssueMessage(IssueOrPullRequestMessage):
    object_type = "issue"


class IssueAssignedAddedV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an issue is assigned.
    """

    topic = "pagure.issue.assigned.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
        },
        "required": ["agent", "project", "issue"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Issue: {fullname}#{id} assigned to {assignee}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
            assignee=self.body["issue"]["assignee"]["name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} assigned issue {name}#{id} to {assignee}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            assignee=self.body["issue"]["assignee"]["name"],
        )


class IssueAssignedResetV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an issue is un-assigned.
    """

    topic = "pagure.issue.assigned.reset"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
        },
        "required": ["agent", "project", "issue"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Issue un-assigned: {fullname}#{id}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} reset the assignee on issue {name}#{id}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
        )


class IssueCommentAddedV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a comment is added to an issue.
    """

    topic = "pagure.issue.comment.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
        },
        "required": ["agent", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Issue: {fullname}#{id} has a new comment\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} commented on the issue {name}#{id}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
        )

    @property
    def url(self):
        issue_url = self.body["issue"]["full_url"]
        commentid = self.body["issue"]["comments"][-1]["id"]

        return "{issue_url}#comment-{commentid}".format(
            issue_url=issue_url, commentid=commentid
        )


class IssueDependencyAddedV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a dependency is added to an issue.
    """

    topic = "pagure.issue.dependency.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
            "added_dependency": {"type": "number"},
        },
        "required": ["agent", "project", "added_dependency"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Issue: {fullname}#{id} depends on #{depissueid}\nBy: {agent_name}".format(
                fullname=self.body["project"]["fullname"],
                id=self.body["issue"]["id"],
                agent_name=self.agent_name,
                depissueid=self.body["added_dependency"],
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} set the issue {name}#{id} as depending on #{depissueid}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            depissueid=self.body["added_dependency"],
        )


class IssueDependencyRemovedV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an issue is deleted.
    """

    topic = "pagure.issue.dependency.removed"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
            "removed_dependency": {"type": "array", "items": {"type": "number"}},
        },
        "required": ["agent", "project", "removed_dependency"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Issue: {fullname}#{id} no longer depending"
            " on #{depissueid}\nBy: {agent_name}".format(
                fullname=self.body["project"]["fullname"],
                id=self.body["issue"]["id"],
                agent_name=self.agent_name,
                depissueid=", #".join(
                    [str(i) for i in self.body["removed_dependency"]]
                ),
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "{agent_name} removed the dependency"
            " on #{depissueid} on the issue {name}#{id}".format(
                agent_name=self.agent_name,
                name=self.body["project"]["fullname"],
                id=self.body["issue"]["id"],
                depissueid=", #".join(
                    [str(i) for i in self.body["removed_dependency"]]
                ),
            )
        )


class IssueDropV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an issue is deleted.
    """

    topic = "pagure.issue.drop"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
        },
        "required": ["agent", "project", "issue"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Issue deleted: {fullname}#{id}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} deleted issue {name}#{id}: {title}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            title=self.body["issue"]["title"],
        )

    @property
    def url(self):
        full_url = self.body["project"]["full_url"]

        return "{full_url}/issues".format(full_url=full_url)


class IssueEditV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an issue is updated.
    """

    topic = "pagure.issue.edit"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
            "fields": {"type": "array", "items": {"type": ["string", "null"]}},
        },
        "required": ["agent", "project", "issue", "fields"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Edited Issue: {fullname}#{id}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "{agent_name} edited fields {fields} of issue {name}#{id}: {title}".format(
                agent_name=self.agent_name,
                name=self.body["project"]["fullname"],
                id=self.body["issue"]["id"],
                title=self.body["issue"]["title"],
                fields=", ".join(self.body["fields"]),
            )
        )


class IssueNewV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.issue.new"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
        },
        "required": ["agent", "project", "issue"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "New Issue: {fullname}#{id}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} created issue {name}#{id}: {title}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            title=self.body["issue"]["title"],
        )


class IssueTagAddedV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an issue is deleted.
    """

    topic = "pagure.issue.tag.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
            "tags": {"type": "array", "items": {"type": "string"}},
        },
        "required": ["agent", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Issue: {fullname}#{id} tagged with {tags}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
            tags=", ".join(self.body["tags"]),
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} tagged the issue {name}#{id} with: {tags}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            tags=", ".join(self.body["tags"]),
        )


class IssueTagRemovedV1(IssueMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when an issue is deleted.
    """

    topic = "pagure.issue.tag.removed"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "issue": ISSUE,
            "tags": {"type": "array", "items": {"type": "string"}},
        },
        "required": ["agent", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Issue: {fullname}#{id} un-tagged with {tags}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            agent_name=self.agent_name,
            tags=", ".join(self.body["tags"]),
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} removed tags {tags} from issue {name}#{id}".format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
            id=self.body["issue"]["id"],
            tags=", ".join(self.body["tags"]),
        )



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Unit tests for common properties of the message schemas."""
import pytest

from pagure_messages.project_schema import ProjectNewV1

from .utils import PROJECT


def test_properties():
    """Assert some properties are correct."""
    body = {
        "agent": "dummy-user",
        "project": PROJECT,
    }
    message = ProjectNewV1(body=body)

    assert message.app_name == "Pagure"
    assert message.app_icon == "https://apps.fedoraproject.org/img/icons/pagure.png"
    assert message.agent_name == "dummy-user"
    assert message.agent_avatar == (
        "https://seccdn.libravatar.org/avatar/"
        "18e8268125372e35f95ef082fd124e9274d46916efe2277417fa5fecfee31af1"
        "?s=64&d=retro"
    )
    assert message.usernames == ["dummy-user"]
    with pytest.warns(DeprecationWarning) as w:
        assert message.agent == "dummy-user"
        assert len(w) == 1
        assert (
            w[0].message.args[0]
            == "agent property is deprecated, please use agent_name instead"
        )



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
import warnings

from fedora_messaging import message
from fedora_messaging.schema_utils import user_avatar_url


SCHEMA_URL = "http://fedoraproject.org/message-schema/"


TagColored = {
    "type": "object",
    "properties": {
        "tag": {"type": "string"},
        "tag_description": {"type": "string"},
        "tag_color": {"type": "string"},
    },
    "required": ["tag", "tag_description", "tag_color"],
}


BOARD = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "active": {"type": "boolean"},
        "status": {"type": ["array", "null"], "items": {"type": "string"}},
        "tag": {"type": TagColored},
        "full_url": {"type": "string"},
    },
    "required": ["name", "active", "status", "tag", "full_url"],
}


BOARD_STATUS = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "bg_color": {"type": "string"},
        "close": {"type": "boolean"},
        "close_status": {"type": ["string", "null"]},
        "default": {"type": "boolean"},
    },
    "required": ["name", "bg_color", "close", "close_status", "default"],
}


BOARD_ISSUE = {
    "type": "object",
    "properties": {
        "board": {"type": BOARD},
        "status": {"type": BOARD_STATUS},
        "rank": {"type": "number"},
    },
    "required": ["board", "rank", "status"],
}


USER = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "fullname": {"type": "string"},
        "url_path": {"type": "string"},
        "full_path": {"type": "string"},
    },
    "required": ["name", "fullname", "url_path"],
}


GIT_RECEIVE_USER = {
    "type": "object",
    "properties": {
        "name": {"oneOf": [{"type": "null"}, {"type": "string"}]},
        "fullname": {"type": "string"},
        "url_path": {"oneOf": [{"type": "null"}, {"type": "string"}]},
    },
    "required": ["name", "fullname", "url_path"],
}


PAGURE_LOG = {
    "type": "object",
    "properties": {
        "id": {"type": "number"},
        "type": {"type": "string"},
        "ref_id": {"type": "string"},
        "date": {"type": "string"},
        "date_created": {"type": "string"},
        "user": USER,
    },
    "required": ["id", "type", "ref_id", "date", "date_created", "user"],
}


MILESTONES = {
    "type": "object",
    "properties": {
        "date": {"type": "string"},
        "active": {"type": "boolean"},
    },
}


PRIORITIES = {
    "type": "object",
    "properties": {
        "date": {"type": "string"},
        "active": {"type": "boolean"},
    },
}


BASE_PROJECT = {
    "type": "object",
    "properties": {
        "id": {"type": "number"},
        "name": {"type": "string"},
        "fullname": {"type": "string"},
        "url_path": {"type": "string"},
        "description": {"type": "string"},
        "namespace": {"type": ["string", "null"]},
        "parent": {"type": "null"},
        "date_created": {"type": "string"},
        "date_modified": {"type": "string"},
        "user": USER,
        # "access_users": {"type": "string"},
        # "access_groups": {"type": "string"},
        "tags": {"type": "array", "items": {"type": "string"}},
        # "priorities": {"type": "object"},
        # "custom_keys": {
        # "type": "array",
        # "items": {"type": "array", "items": {"type": "string"}},
        # },
        "close_status": {"type": "array", "items": {"type": "string"}},
        "milestones": {"type": "object", "properties": {"type": MILESTONES}},
    },
    "required": [
        "id",
        "name",
        "fullname",
        "url_path",
        "description",
        "namespace",
        "parent",
        "date_created",
        "date_modified",
        "user",
        # "access_users",
        # "access_groups",
        "tags",
        # "priorities",
        # "custom_keys",
        "close_status",
        "milestones",
    ],
}


PROJECT = {
    "type": "object",
    "properties": {
        "id": {"type": "number"},
        "name": {"type": "string"},
        "fullname": {"type": "string"},
        "url_path": {"type": "string"},
        "full_url": {"type": "string"},
        "description": {"type": "string"},
        "namespace": {"type": ["string", "null"]},
        # "parent": {"type": ["null", BASE_PROJECT]},
        "date_created": {"type": "string"},
        "date_modified": {"type": "string"},
        "user": USER,
        # "access_users": {"type": "string"},
        # "access_groups": {"type": "string"},
        "tags": {"type": "array", "items": {"type": "string"}},
        # "priorities": {"type": "object"},
        # "custom_keys": {
        # "type": "array",
        # "items": {"type": "array", "items": {"type": "string"}},
        # },
        "close_status": {"type": "array", "items": {"type": "string"}},
        "milestones": {
            "oneOf": [
                {"type": "null"},
                MILESTONES,
            ]
        },
    },
    "required": [
        "id",
        "name",
        "full_url",
        "fullname",
        "url_path",
        "description",
        "namespace",
        # "parent",
        "date_created",
        "date_modified",
        "user",
        # "access_users",
        # "access_groups",
        "tags",
        # "priorities",
        # "custom_keys",
        "close_status",
        "milestones",
    ],
}


RELATED_PR = {
    "type": "object",
    "properties": {
        "id": {"type": "number"},
        "title": {"type": "string"},
    },
    "required": ["id", "title"],
}


ISSUE = {
    "type": "object",
    "properties": {
        "id": {"type": "number"},
        "full_url": {"type": "string"},
        "title": {"type": "string"},
        "content": {"type": "string"},
        "status": {"type": "string"},
        "close_status": {"type": ["string", "null"]},
        "date_created": {"type": "string"},
        "last_updated": {"type": "string"},
        "closed_at": {"type": ["string", "null"]},
        "user": USER,
        "private": {"type": "boolean"},
        "tags": {"type": "array", "items": {"type": "string"}},
        "depends": {"type": "array", "items": {"type": "string"}},
        "blocks": {"type": "array", "items": {"type": "string"}},
        "assignee": {"oneOf": [{"type": "null"}, USER]},
        # "priorities": {"type": "object"},
        "milestone": {
            "oneOf": [
                {"type": "null"},
                {"type": "string"},
            ]
        },
        # "custom_fields": {
        # "type": "array",
        # "items": {"type": "array", "items": {"type": "string"}},
        # },
        "closed_by": {"oneOf": [{"type": "null"}, USER]},
        "related_prs": {
            "oneOf": [{"type": "null"}, {"type": "array", "items": RELATED_PR}]
        },
    },
    "required": [
        "id",
        "full_url",
        "title",
        "content",
        "status",
        "close_status",
        "date_created",
        "last_updated",
        "closed_at",
        "user",
        "private",
        "tags",
        "depends",
        "blocks",
        "assignee",
        # "priorities",
        "milestone",
        # "custom_fields",
        "closed_by",
        "related_prs",
    ],
}


PULL_REQUEST = {
    "type": "object",
    "properties": {
        "id": {"type": "number"},
        "uid": {"type": "string"},
        "title": {"type": "string"},
        "full_url": {"type": "string"},
        "branch": {"type": "string"},
        "project": PROJECT,
        "branch_from": {"type": "string"},
        "repo_from": {"oneOf": [{"type": "null"}, USER]},
        "remote_git": {"oneOf": [{"type": "null"}, {"type": "string"}]},
        "date_created": {"type": "string"},
        "updated_on": {"type": "string"},
        "last_updated": {"type": "string"},
        "closed_at": {"oneOf": [{"type": "null"}, {"type": "string"}]},
        "user": USER,
        "assignee": {"oneOf": [{"type": "null"}, USER]},
        "status": {"type": "string"},
        "commit_start": {"oneOf": [{"type": "null"}, {"type": "string"}]},
        "commit_stop": {"oneOf": [{"type": "null"}, {"type": "string"}]},
        "closed_by": {"oneOf": [{"type": "null"}, USER]},
        "initial_comment": {"oneOf": [{"type": "null"}, {"type": "string"}]},
        "cached_merge_status": {"type": "string"},
        "threshold_reached": {"oneOf": [{"type": "null"}, {"type": "boolean"}]},
        "tags": {"type": "array", "items": {"type": "string"}},
    },
    "required": [
        "id",
        "uid",
        "title",
        "full_url",
        "branch",
        "project",
        "branch_from",
        "repo_from",
        "remote_git",
        "date_created",
        "updated_on",
        "last_updated",
        "closed_at",
        "user",
        "assignee",
        "status",
        "commit_start",
        "commit_stop",
        "closed_by",
        "initial_comment",
        "cached_merge_status",
        "threshold_reached",
        "tags",
    ],
}


COMMIT_FLAG = {
    "type": "object",
    "properties": {
        "commit_hash": {"type": "string"},
        "username": {"type": "string"},
        "comment": {"type": "string"},
        "status": {"type": "string"},
        "url": {"type": "string"},
        "date_created": {"type": "string"},
        "date_updated": {"type": "string"},
        "user": USER,
    },
    "required": [
        "commit_hash",
        "username",
        "comment",
        "status",
        "url",
        "date_created",
        "date_updated",
        "user",
    ],
}


GROUP = {
    "type": "object",
    "properties": {
        "display_name": {"type": "string"},
        "description": {"type": "string"},
        "creator": USER,
        "members": {"type": "array", "items": {"type": "string"}},
        "date_created": {"type": "string"},
        "group_type": {"type": "string"},
        "name": {"type": "string"},
        "full_url": {"type": "string"},
    },
    "required": [
        "display_name",
        "description",
        "creator",
        "members",
        "date_created",
        "group_type",
        "name",
        "full_url",
    ],
}


def pretty_list(value):
    """Return a de-duplicated, sorted, and None-filtered list."""
    value = list(set(val for val in value if val is not None))
    value.sort()
    return value


class PagureMessage(message.Message):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure.
    """

    __link__ = "https://pagure.io"
    __stg_link__ = "https://stg.pagure.io"

    def get_base_url(self):
        base_url = self.__link__
        if ".stg." in self.topic:  # pragma: no cover
            base_url = self.__stg_link__
        return base_url

    @property
    def app_name(self):
        return "Pagure"

    @property
    def app_icon(self):
        return "https://apps.fedoraproject.org/img/icons/pagure.png"

    @property
    def agent(self):
        warnings.warn(
            "agent property is deprecated, please use agent_name instead",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.agent_name

    @property
    def agent_name(self):
        return self.body.get("agent")

    @property
    def agent_avatar(self):
        return user_avatar_url(self.agent_name)

    @property
    def usernames(self):
        return [self.agent_name]


class IssueOrPullRequestMessage(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure about an issue or a pull request.
    """

    object_type = None

    @property
    def url(self):
        return self.body[self.object_type]["full_url"]

    @property
    def usernames(self):
        usernames = [
            self.agent_name,
            self.body[self.object_type]["user"]["name"],
        ]
        try:
            usernames.append(self.body[self.object_type]["assignee"]["name"])
        except (KeyError, TypeError):
            pass
        return pretty_list(usernames)



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from fedora_messaging import message

import pkg_resources


from .git_schema import (  # noqa: F401
    GitBranchCreationV1,
    GitBranchDeletionV1,
    GitReceiveV1,
    GitTagCreationV1,
    GitTagDeletionV1,
)
from .issue_schema import (  # noqa: F401
    IssueAssignedAddedV1,
    IssueAssignedResetV1,
    IssueCommentAddedV1,
    IssueDependencyAddedV1,
    IssueDependencyRemovedV1,
    IssueDropV1,
    IssueEditV1,
    IssueNewV1,
    IssueTagAddedV1,
    IssueTagRemovedV1,
)
from .misc_schema import (  # noqa: F401
    CommitFlagAddedV1,
    CommitFlagUpdatedV1,
    GroupEditV1,
    TestNotificationV1,
)
from .project_schema import (  # noqa: F401
    ProjectDeletedV1,
    ProjectEditV1,
    ProjectForkedV1,
    ProjectGroupAccessUpdatedV1,
    ProjectGroupAddedV1,
    ProjectGroupRemovedV1,
    ProjectNewV1,
    ProjectTagEditedV1,
    ProjectTagRemovedV1,
    ProjectUserAccessUpdatedV1,
    ProjectUserAddedV1,
    ProjectUserRemovedV1,
)
from .pull_requests_schema import (  # noqa: F401
    PullRequestAssignedAddedV1,
    PullRequestAssignedResetV1,
    PullRequestClosedV1,
    PullRequestCommentAddedV1,
    PullRequestCommentEditedV1,
    PullRequestFlagAddedV1,
    PullRequestFlagUpdatedV1,
    PullRequestInitialCommentEditedV1,
    PullRequestNewV1,
    PullRequestRebasedV1,
    PullRequestReopenedV1,
    PullRequestTagAddedV1,
    PullRequestTagRemovedV1,
    PullRequestUpdatedV1,
)


def get_message_object_from_topic(topic):
    """Returns the Message class corresponding to the topic."""

    output = None

    for entry_point in pkg_resources.iter_entry_points("fedora.messages"):
        cls = entry_point.load()
        if cls.topic == topic:
            output = cls
            break

    if output is None:
        output = message.Message

    return output



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from .base import PROJECT, PagureMessage, SCHEMA_URL


class ProjectNewV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.project.new"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
        },
        "required": ["agent", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "New Project: {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} created project "{name}"'.format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectEditV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.project.edit"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "fields": {"type": "array", "items": {"type": ["string", "null"]}},
        },
        "required": ["agent", "project", "fields"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Project Edited: {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} edited the fields {fields} of project "{name}"'.format(
            agent_name=self.agent_name,
            fields=", ".join(self.body["fields"]),
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectForkedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.project.forked"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
        },
        "required": ["agent", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Project: {fullname}\nForked by: {agent_name}".format(
            fullname=self.body["project"]["parent"]["fullname"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} forked project "{parent}" to {name}'.format(
            agent_name=self.agent_name,
            parent=self.body["project"]["parent"]["fullname"],
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectDeletedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.project.deleted"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
        },
        "required": ["agent", "project"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Project: {fullname}\nDeleted by: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} deleted project "{name}"'.format(
            agent_name=self.agent_name,
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectGroupAddedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.

    Example: https://apps.fedoraproject.org/datagrepper/v2/id?id=d9d4483c-f1a2-4d51-89ac-a3345802f929&is_raw=true&size=extra-large
    """

    topic = "pagure.project.group.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "new_group": {"type": "string"},
            "access": {"type": "string"},
            "branches": {"type": ["string", "null"]},
        },
        "required": ["agent", "project", "new_group", "access", "branches"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Group: {group} added to {fullname} as {access}\nBy: {agent_name}".format(
                fullname=self.body["project"]["fullname"],
                group=self.body["new_group"],
                access=self.body["access"],
                agent_name=self.agent_name,
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            '{agent_name} added the group {group} to the project "{name}" at '
            "the {access} level".format(
                agent_name=self.agent_name,
                group=self.body["new_group"],
                access=self.body["access"],
                name=self.body["project"]["fullname"],
            )
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectGroupRemovedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.

    Example: https://apps.fedoraproject.org/datagrepper/v2/id?id=bd1fdfe6-f64e-436b-a148-14043651d511&is_raw=true&size=extra-large
    """

    topic = "pagure.project.group.removed"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "removed_groups": {"type": "array", "items": {"type": "string"}},
        },
        "required": ["agent", "project", "removed_groups"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Group(s) {groups} removed from {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            groups=", ".join(self.body["removed_groups"]),
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "{agent_name} removed the group(s) {groups} from the "
            'project "{name}"'.format(
                agent_name=self.agent_name,
                groups=", ".join(self.body["removed_groups"]),
                name=self.body["project"]["fullname"],
            )
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectGroupAccessUpdatedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.

    Example: https://apps.fedoraproject.org/datagrepper/v2/id?id=4bd92c2d-59a3-4e4d-b4c6-eaa4f504cff0&is_raw=true&size=extra-large
    """

    topic = "pagure.project.group.access.updated"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "new_group": {"type": "string"},
            "new_access": {"type": "string"},
            "branches": {"type": ["string", "null"]},
        },
        "required": ["agent", "project", "new_group", "new_access", "branches"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Group: {group} access updated to {access} on {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            group=self.body["new_group"],
            access=self.body["new_access"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "{agent_name} updated the access of group {group} to {access} on "
            'the project "{name}"'.format(
                agent_name=self.agent_name,
                group=self.body["new_group"],
                access=self.body["new_access"],
                name=self.body["project"]["fullname"],
            )
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectTagEditedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.project.tag.edited"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "old_tag": {"type": "string"},
            "old_tag_description": {"type": "string"},
            "old_tag_color": {"type": "string"},
            "new_tag": {"type": "string"},
            "new_tag_description": {"type": "string"},
            "new_tag_color": {"type": "string"},
        },
        "required": [
            "agent",
            "project",
            "old_tag",
            "old_tag_description",
            "old_tag_color",
            "new_tag",
            "new_tag_description",
            "new_tag_color",
        ],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Tag: {tag_name} edited on {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            tag_name=self.body["new_tag"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} edited the tag {tag_name} on the project "{name}"'.format(
            agent_name=self.agent_name,
            tag_name=self.body["new_tag"],
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectTagRemovedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.project.tag.removed"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "tags": {"type": "array", "items": {"type": "string"}},
        },
        "required": ["agent", "project", "tags"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Tag(s): {tags} removed from {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            tags=", ".join(self.body["tags"]),
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} removed the tag(s) {tags} of project "{name}"'.format(
            agent_name=self.agent_name,
            tags=", ".join(self.body["tags"]),
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectUserAccessUpdatedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.

    Example: https://apps.fedoraproject.org/datagrepper/v2/id?id=ab665d5b-766e-4521-960c-b30d9f223011&is_raw=true&size=extra-large
    """

    topic = "pagure.project.user.access.updated"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "new_user": {"type": "string"},
            "new_access": {"type": "string"},
            "new_branches": {"type": ["string", "null"]},
        },
        "required": ["agent", "project", "new_user", "new_access", "new_branches"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "User: {user} access edited to {new_access} on {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            user=self.body["new_user"],
            new_access=self.body["new_access"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "{agent_name} updated the access of {user} to {new_access} on the "
            'project "{name}"'.format(
                agent_name=self.agent_name,
                user=self.body["new_user"],
                new_access=self.body["new_access"],
                name=self.body["project"]["fullname"],
            )
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectUserAddedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.

    Example: https://apps.fedoraproject.org/datagrepper/v2/id?id=4323d2ff-2cdc-462f-b775-51d3a7fb68e9&is_raw=true&size=extra-large
    """

    topic = "pagure.project.user.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "new_user": {"type": "string"},
            "access": {"type": "string"},
            "branches": {"type": ["string", "null"]},
        },
        "required": ["agent", "project", "new_user", "access", "branches"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "User: {user} added to {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            user=self.body["new_user"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} added the {user} to the project "{name}"'.format(
            agent_name=self.agent_name,
            user=self.body["new_user"],
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]


class ProjectUserRemovedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.project.user.removed"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "project": PROJECT,
            "removed_user": {"type": "string"},
        },
        "required": ["agent", "project", "removed_user"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "User: {user} removed from {fullname}\nBy: {agent_name}".format(
            fullname=self.body["project"]["fullname"],
            user=self.body["removed_user"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return '{agent_name} removed the {user} from the project "{name}"'.format(
            agent_name=self.agent_name,
            user=self.body["removed_user"],
            name=self.body["project"]["fullname"],
        )

    @property
    def url(self):
        return self.body["project"]["full_url"]



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Unit tests for common properties of the message schemas."""

from pagure_messages import get_message_object_from_topic

from .utils import PROJECT


def test_object_type():
    """Assert some properties are correct."""
    cls = get_message_object_from_topic("pagure.git.receive")
    msg = cls(body={"repo": PROJECT})
    assert str(type(msg)) == "<class 'pagure_messages.git_schema.GitReceiveV1'>"


def test_topic():
    """Assert some properties are correct."""
    cls = get_message_object_from_topic("pagure.git.receive")
    msg = cls(body={"repo": PROJECT})

    assert msg.topic == "pagure.git.receive"


def test_invalidtopic():
    """Assert some properties are correct."""
    cls = get_message_object_from_topic("fedocal.invalid.topic")

    assert str(type(cls())) == "<class 'fedora_messaging.message.Message'>"



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from .base import GIT_RECEIVE_USER, PROJECT, PagureMessage, SCHEMA_URL


class GitMessage(PagureMessage):
    """
    Used when git events generate Fedora messages.
    """

    def _name_if_namespace(self, namespace):
        if self.body["repo"]["namespace"] == namespace:
            return [self.body["repo"]["name"]]
        return []

    @property
    def packages(self):
        return self._name_if_namespace("rpms")

    @property
    def containers(self):
        return self._name_if_namespace("containers")

    @property
    def modules(self):
        return self._name_if_namespace("modules")

    @property
    def flatpaks(self):
        return self._name_if_namespace("flatpaks")


class GitBranchCreationV1(GitMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.git.branch.creation"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "repo": PROJECT,
            "tag": {"type": "string"},
            "rev": {"type": "string"},
            "authors": {"type": "array", "items": GIT_RECEIVE_USER},
        },
        "required": ["agent", "repo", "branch", "rev", "authors"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Git branch: {branch} created\nBy: {agent_name}".format(
            branch=self.body["branch"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} created the branch {branch} on {name}".format(
            agent_name=self.agent_name,
            name=self.body["repo"]["fullname"],
            branch=self.body["branch"],
        )

    @property
    def url(self):
        base_url = self.body["repo"]["full_url"]

        item = self.body["branch"]
        if "refs/heads/" in item:
            item = item.replace("refs/heads/", "")

        tmpl = "{base_url}/tree/{item}"
        return tmpl.format(base_url=base_url, item=item)


class GitBranchDeletionV1(GitMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.git.branch.deletion"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "repo": PROJECT,
            "tag": {"type": "string"},
            "rev": {"type": "string"},
            "authors": {"type": "array", "items": GIT_RECEIVE_USER},
        },
        "required": ["agent", "repo", "branch", "rev", "authors"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Git branch: {branch} deleted\nBy: {agent_name}".format(
            branch=self.body["branch"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} deleted the branch {branch} on {name}".format(
            agent_name=self.agent_name,
            name=self.body["repo"]["fullname"],
            branch=self.body["branch"],
        )

    @property
    def url(self):
        return self.body["repo"]["full_url"]


class GitReceiveV1(GitMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.git.receive"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "forced": {"type": "boolean"},
            "repo": PROJECT,
            "old_commit": {"type": "string"},
            "branch": {"type": "string"},
            "authors": {"type": "array", "items": GIT_RECEIVE_USER},
            "total_commits": {"type": "number"},
            "start_commit": {"type": "string"},
            "end_commit": {"type": "string"},
        },
        "required": [
            "agent",
            "forced",
            "repo",
            "old_commit",
            "branch",
            "authors",
            "total_commits",
            "start_commit",
            "end_commit",
        ],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        author = self.body["authors"][0]
        force_prefix = "forced-" if self.body["forced"] else ""
        plural_suffix = "s" if self.body["total_commits"] > 1 else ""
        branch_short = self.body["branch"].replace("refs/heads/", "")
        tpl = (
            "{author[fullname]} ({author[name]}) {force_prefix}pushed {total_commits} "
            "commit{plural_suffix} on {repo[fullname]}.\n"
            "Branch: {branch_short}\n"
        )
        return tpl.format(
            **self.body,
            author=author,
            force_prefix=force_prefix,
            plural_suffix=plural_suffix,
            branch_short=branch_short,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        branch = self.body["branch"].replace("refs/heads/", "")
        return "{agent_name} pushed {count} commits on {fullname} (branch: {branch})".format(
            agent_name=self.agent_name,
            fullname=self.body["repo"]["fullname"],
            count=self.body["total_commits"],
            branch=branch,
        )

    @property
    def url(self):
        if self.body["total_commits"] == 1:
            return "{base_url}/c/{rev}".format(
                base_url=self.body["repo"]["full_url"], rev=self.body["end_commit"]
            )
        else:
            return "{base_url}/c/{old}..{end}".format(
                base_url=self.body["repo"]["full_url"],
                old=self.body["old_commit"],
                end=self.body["end_commit"],
            )

    @property
    def patch_url(self):
        return f"{self.url}.patch" if self.body["total_commits"] == 1 else None


class GitTagCreationV1(GitMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.git.tag.creation"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "repo": PROJECT,
            "tag": {"type": "string"},
            "rev": {"type": "string"},
            "authors": {"type": "array", "items": GIT_RECEIVE_USER},
        },
        "required": ["agent", "repo", "tag", "rev", "authors"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Git tag: {tag} created\nBy: {agent_name}".format(
            tag=self.body["tag"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} tagged the commit {rev} on {name} as {tag}".format(
            agent_name=self.agent_name,
            name=self.body["repo"]["fullname"],
            tag=self.body["tag"],
            rev=self.body["rev"],
        )

    @property
    def url(self):
        base_url = self.body["repo"]["full_url"]
        tag = self.body["tag"]

        tmpl = "{base_url}/commits/{tag}"
        return tmpl.format(base_url=base_url, tag=tag)


class GitTagDeletionV1(GitMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.git.tag.deletion"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "repo": PROJECT,
            "tag": {"type": "string"},
            "rev": {"type": "string"},
            "authors": {"type": "array", "items": GIT_RECEIVE_USER},
        },
        "required": ["agent", "repo", "tag", "rev", "authors"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Git tag: {tag} deleted\nBy: {agent_name}".format(
            tag=self.body["tag"],
            agent_name=self.agent_name,
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return "{agent_name} deleted the tag {tag} of commit {rev} on {name}".format(
            agent_name=self.agent_name,
            name=self.body["repo"]["fullname"],
            tag=self.body["tag"],
            rev=self.body["rev"],
        )

    @property
    def url(self):
        base_url = self.body["repo"]["full_url"]
        tmpl = "{base_url}/releases"
        return tmpl.format(base_url=base_url)



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from .base import COMMIT_FLAG, GROUP, PROJECT, PagureMessage, SCHEMA_URL


class CommitFlagAddedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.commit.flag.added"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "repo": PROJECT,
            "flag": COMMIT_FLAG,
        },
        "required": ["agent", "repo", "flag"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "New commit flag: {username} {status}\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            username=self.body["flag"]["username"],
            status=self.body["flag"]["status"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "Commit {commit_hash} of project {name} was flagged as {status} "
            "by {username}".format(
                name=self.body["repo"]["fullname"],
                commit_hash=self.body["flag"]["commit_hash"],
                username=self.body["flag"]["username"],
                status=self.body["flag"]["status"],
            )
        )

    @property
    def url(self):
        full_url = self.body["repo"]["full_url"]
        commithash = self.body["flag"]["commit_hash"]

        return "{full_url}/c/{commithash}".format(
            full_url=full_url, commithash=commithash
        )


class CommitFlagUpdatedV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.commit.flag.updated"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "repo": PROJECT,
            "flag": COMMIT_FLAG,
        },
        "required": ["agent", "repo", "flag"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "New commit flag: {username} {status}\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            username=self.body["flag"]["username"],
            status=self.body["flag"]["status"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "{username} updated flag on commit {commit_hash} of project "
            "{name} to {status}".format(
                name=self.body["repo"]["fullname"],
                commit_hash=self.body["flag"]["commit_hash"],
                username=self.body["flag"]["username"],
                status=self.body["flag"]["status"],
            )
        )

    @property
    def url(self):
        full_url = self.body["repo"]["full_url"]
        commithash = self.body["flag"]["commit_hash"]

        return "{full_url}/c/{commithash}".format(
            full_url=full_url, commithash=commithash
        )


class GroupEditV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.group.edit"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "group": GROUP,
            "fields": {"type": "array", "items": {"type": "string"}},
        },
        "required": ["agent", "group", "fields"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Group edit: {group_name}\nBy: {agent_name}".format(
            agent_name=self.agent_name,
            group_name=self.body["group"]["name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "{agent_name} edited the fields {fields} of the group {group_name}".format(
                agent_name=self.agent_name,
                group_name=self.body["group"]["name"],
                fields=", ".join(self.body["fields"]),
            )
        )

    @property
    def url(self):
        return self.body["group"]["full_url"]


class TestNotificationV1(PagureMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by pagure when a new thing is created.
    """

    topic = "pagure.Test.notification"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a new project is created",
        "type": "object",
        "properties": {
            "content": {"type": "string"},
        },
        "required": ["content"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "Test notification"

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)

    @property
    def url(self):
        return self.get_base_url()



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Unit tests for the message schema."""

from jsonschema import ValidationError

import pytest

from pagure_messages.project_schema import ProjectUserAccessUpdatedV1

from .utils import PROJECT


def test_minimal():
    """
    Assert the message schema validates a message with the required fields.
    """
    body = {
        "agent": "dummy-user",
        "project": PROJECT,
        "new_user": "newu",
        "new_access": "commit",
        "new_branches": None,
    }
    message = ProjectUserAccessUpdatedV1(body=body)
    message.validate()
    assert message.url == "http://localhost.localdomain/fedora-infra/fedocal-messages"


def test_missing_fields():
    """Assert an exception is actually raised on validation failure."""
    minimal_message = {
        "agent": "dummy-user",
        "project": PROJECT,
    }
    message = ProjectUserAccessUpdatedV1(body=minimal_message)
    with pytest.raises(ValidationError):
        message.validate()


def test_str():
    """Assert __str__ produces a human-readable message."""
    body = {
        "agent": "dummy-user",
        "project": PROJECT,
        "new_user": "newu",
        "new_access": "commit",
        "new_branches": None,
    }
    expected_str = (
        "User: newu access edited to commit on "
        "fedora-infra/fedocal-messages\nBy: dummy-user"
    )
    message = ProjectUserAccessUpdatedV1(body=body)
    message.validate()
    assert expected_str == str(message)


def test_summary():
    """Assert the summary is correct."""
    body = {
        "agent": "dummy-user",
        "project": PROJECT,
        "new_user": "newu",
        "new_access": "commit",
        "new_branches": None,
    }
    expected_summary = (
        "dummy-user updated the access of newu to commit on "
        'the project "fedora-infra/fedocal-messages"'
    )
    message = ProjectUserAccessUpdatedV1(body=body)
    message.validate()
    assert expected_summary == message.summary
